@@ -54,7 +54,6 @@ else:
     from contextlib2 import ExitStack  # NOQA
     PermissionError = OSError
 
-
 on_win = (sys.platform == 'win32')
 
 codec = getpreferredencoding() or 'utf-8'
@@ -77,6 +76,207 @@ if __name__ == '__main__':
     sys.exit(%(func)s())
 """
 
+try:
+    from os import scandir, walk  # NOQA
+except ImportError:
+    from scandir import walk
+
+
+@memoized
+def stat_file(path):
+    return os.stat(path)
+
+
+def directory_size(path):
+    '''
+    '''
+    total_size = 0
+    seen = set()
+
+    for root, _, files in walk(path):
+        for f in files:
+            try:
+                stat = stat_file(os.path.join(root, f))
+            except OSError:
+                continue
+
+            if stat.st_ino in seen:
+                continue
+
+            seen.add(stat.st_ino)
+
+            total_size += stat.st_size
+    return total_size  # size in bytes
+
+
+class DummyPsutilProcess(object):
+    def children(self, *args, **kwargs):
+        return []
+
+
+class PopenWrapper(object):
+    # Small wrapper around subprocess.Popen to allow memory usage monitoring
+    # copied from ProtoCI, https://github.com/ContinuumIO/ProtoCI/blob/59159bc2c9f991fbfa5e398b6bb066d7417583ec/protoci/build2.py#L20  # NOQA
+
+    def __init__(self, *args, **kwargs):
+        self.elapsed = None
+        self.rss = 0
+        self.vms = 0
+        self.returncode = None
+        self.disk = 0
+        self.processes = 1
+
+        self.out, self.err = self._execute(*args, **kwargs)
+
+    def _execute(self, *args, **kwargs):
+        try:
+            import psutil
+            psutil_exceptions = psutil.NoSuchProcess, psutil.AccessDenied, psutil.NoSuchProcess
+        except ImportError as e:
+            psutil = None
+            psutil_exceptions = (OSError, ValueError)
+            log = get_logger(__name__)
+            log.warn("psutil import failed.  Error was {}".format(e))
+            log.warn("only disk usage and time statistics will be available.  Install psutil to "
+                     "get CPU time and memory usage statistics.")
+
+        # The polling interval (in seconds)
+        time_int = kwargs.pop('time_int', 1)
+
+        disk_usage_dir = kwargs.get('cwd', sys.prefix)
+
+        # Create a process of this (the parent) process
+        parent = psutil.Process(os.getpid()) if psutil else DummyPsutilProcess()
+
+        cpu_usage = defaultdict(dict)
+
+        # Using the convenience Popen class provided by psutil
+        start_time = time.time()
+        _popen = psutil.Popen(*args, **kwargs) if psutil else subprocess.Popen(*args, **kwargs)
+        try:
+            while self.returncode is None:
+                # We need to get all of the children of our process since our
+                # process spawns other processes.  Collect all of the child
+                # processes
+
+                rss = 0
+                vms = 0
+                processes = 0
+                # We use the parent process to get mem usage of all spawned processes
+                for child in parent.children(recursive=True):
+                    child_cpu_usage = cpu_usage.get(child.pid, {})
+                    try:
+                        mem = child.memory_info()
+                        rss += mem.rss
+                        vms += mem.rss
+                        # listing child times are only available on linux, so we don't use them.
+                        #    we are instead looping over children and getting each individually.
+                        #    https://psutil.readthedocs.io/en/latest/#psutil.Process.cpu_times
+                        cpu_stats = child.cpu_times()
+                        child_cpu_usage['sys'] = cpu_stats.system
+                        child_cpu_usage['user'] = cpu_stats.user
+                        cpu_usage[child.pid] = child_cpu_usage
+                    except psutil_exceptions:
+                        # process already died.  Just ignore it.
+                        continue
+                    processes += 1
+
+                # Sum the memory usage of all the children together (2D columnwise sum)
+                self.rss = max(rss, self.rss)
+                self.vms = max(vms, self.vms)
+                self.cpu_sys = sum(child['sys'] for child in cpu_usage.values())
+                self.cpu_user = sum(child['user'] for child in cpu_usage.values())
+                self.processes = max(processes, self.processes)
+
+                # Get disk usage
+                self.disk = max(directory_size(disk_usage_dir), self.disk)
+
+                time.sleep(time_int)
+                self.elapsed = time.time() - start_time
+                self.returncode = _popen.poll()
+        except KeyboardInterrupt:
+            _popen.kill()
+            raise
+
+        return _popen.stdout, _popen.stderr
+
+    def __repr__(self):
+        return str({'elapsed': self.elapsed,
+                    'rss': self.rss,
+                    'vms': self.vms,
+                    'disk': self.disk,
+                    'processes': self.processes,
+                    'cpu_user': self.cpu_user,
+                    'cpu_sys': self.cpu_sys,
+                    'returncode': self.returncode})
+
+
+def _func_defaulting_env_to_os_environ(func, *popenargs, **kwargs):
+    if 'env' not in kwargs:
+        kwargs = kwargs.copy()
+        env_copy = os.environ.copy()
+        kwargs.update({'env': env_copy})
+    kwargs['env'] = {str(key): str(value) for key, value in kwargs['env'].items()}
+    _args = []
+    if 'stdin' not in kwargs:
+        kwargs['stdin'] = subprocess.PIPE
+    for arg in popenargs:
+        # arguments to subprocess need to be bytestrings
+        if sys.version_info.major < 3 and hasattr(arg, 'encode'):
+            arg = arg.encode(codec)
+        elif sys.version_info.major >= 3 and hasattr(arg, 'decode'):
+            arg = arg.decode(codec)
+        _args.append(str(arg))
+
+    stats = kwargs.get('stats')
+    if stats is not None:
+        del kwargs['stats']
+    proc = PopenWrapper(_args, **kwargs)
+    out = None
+
+    if func == 'output':
+        out = proc.out.read()
+
+    if proc.returncode != 0:
+        raise subprocess.CalledProcessError(proc.returncode, _args)
+
+    if stats is not None:
+        stats.update({'elapsed': proc.elapsed,
+                      'disk': proc.disk,
+                      'processes': proc.processes,
+                      'cpu_user': proc.cpu_user,
+                      'cpu_sys': proc.cpu_sys,
+                      'rss': proc.rss,
+                      'vms': proc.vms})
+    return out
+
+
+def check_call_env(popenargs, **kwargs):
+    return _func_defaulting_env_to_os_environ('call', *popenargs, **kwargs)
+
+
+def check_output_env(popenargs, **kwargs):
+    return _func_defaulting_env_to_os_environ('output', stdout=subprocess.PIPE,
+                                              *popenargs, **kwargs)\
+        .rstrip()
+
+
+def bytes2human(n):
+    # http://code.activestate.com/recipes/578019
+    # >>> bytes2human(10000)
+    # '9.8K'
+    # >>> bytes2human(100001221)
+    # '95.4M'
+    symbols = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
+    prefix = {}
+    for i, s in enumerate(symbols):
+        prefix[s] = 1 << (i + 1) * 10
+    for s in reversed(symbols):
+        if n >= prefix[s]:
+            value = float(n) / prefix[s]
+            return '%.1f%s' % (value, s)
+    return "%sB" % n
+
 
 def get_recipe_abspath(recipe):
     """resolve recipe dir as absolute path.  If recipe is a tarball rather than a folder,
@@ -497,7 +697,7 @@ def safe_print_unicode(*args, **kwargs):
 
 def rec_glob(path, patterns):
     result = []
-    for d_f in os.walk(path):
+    for d_f in walk(path):
         # ignore the .git folder
         # if '.git' in d_f[0]:
         #     continue
@@ -649,40 +849,12 @@ def create_entry_points(items, config):
 
 # Return all files in dir, and all its subdirectories, ending in pattern
 def get_ext_files(start_path, pattern):
-    for root, _, files in os.walk(start_path):
+    for root, _, files in walk(start_path):
         for f in files:
             if f.endswith(pattern):
                 yield os.path.join(root, f)
 
 
-def _func_defaulting_env_to_os_environ(func, *popenargs, **kwargs):
-    if 'env' not in kwargs:
-        kwargs = kwargs.copy()
-        env_copy = os.environ.copy()
-        kwargs.update({'env': env_copy})
-    kwargs['env'] = {str(key): str(value) for key, value in kwargs['env'].items()}
-    _args = []
-    if 'stdin' not in kwargs:
-        kwargs['stdin'] = subprocess.PIPE
-    for arg in popenargs:
-        # arguments to subprocess need to be bytestrings
-        if sys.version_info.major < 3 and hasattr(arg, 'encode'):
-            arg = arg.encode(codec)
-        elif sys.version_info.major >= 3 and hasattr(arg, 'decode'):
-            arg = arg.decode(codec)
-        _args.append(str(arg))
-    return func(_args, **kwargs)
-
-
-def check_call_env(popenargs, **kwargs):
-    return _func_defaulting_env_to_os_environ(subprocess.check_call, *popenargs, **kwargs)
-
-
-def check_output_env(popenargs, **kwargs):
-    return _func_defaulting_env_to_os_environ(subprocess.check_output, *popenargs, **kwargs)\
-        .rstrip()
-
-
 _posix_exes_cache = {}
 
 
@@ -761,7 +933,7 @@ def expand_globs(path_list, root_dir):
         elif os.path.islink(path):
             files.append(path)
         elif os.path.isdir(path):
-            files.extend(os.path.join(root, f) for root, _, fs in os.walk(path) for f in fs)
+            files.extend(os.path.join(root, f) for root, _, fs in walk(path) for f in fs)
         else:
             # File compared to the globs use / as separator indenpendently of the os
             glob_files = glob(path)
@@ -1191,7 +1363,7 @@ def prefix_files(prefix):
     Returns a set of all files in prefix.
     '''
     res = set()
-    for root, dirs, files in os.walk(prefix):
+    for root, dirs, files in walk(prefix):
         for fn in files:
             res.add(join(root, fn)[len(prefix) + 1:])
         for dn in dirs:
